# -*- coding: utf-8
"""Implements Page models"""

from __future__ import unicode_literals

from django.db import models
from django.db.models import Q
from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site
from django.utils.encoding import python_2_unicode_compatible
from django.contrib.contenttypes.models import ContentType

from mptt.models import MPTTModel, TreeForeignKey
from guardian.models import UserObjectPermissionBase
from guardian.models import GroupObjectPermissionBase
from guardian.models import UserObjectPermission
from guardian.models import GroupObjectPermission

from pages.conf import settings
from pages.cache import cache
from pages.managers import PageManager
from pages.models.pagecontenttype import PageContentType
from pages.models import pagecontenttypes


@python_2_unicode_compatible
class Page(MPTTModel):
    name = models.CharField(max_length=200, unique=True)
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
    template = models.CharField(max_length=254, blank=True)
    comment = models.TextField(max_length=254, blank=True)
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='page_creator', null=True)
    updated_by = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='page_editor', null=True)
    date_created = models.DateTimeField(_('Created'), default=timezone.now)
    date_updated = models.DateTimeField(_('Updated'), default=timezone.now)
    date_approved = models.DateTimeField(_('Approved'), null=True, blank=True)
    date_publication = models.DateTimeField(_('Publication date'), null=True, blank=True)
    date_publication_end = models.DateTimeField(_('Publication end date'), null=True, blank=True)
    is_draft = models.BooleanField(_('Draft'), default=True)
    is_approved = models.BooleanField(_('Approved'), default=False)
    is_hidden = models.BooleanField(_('Hidden'), default=False)
    is_published = models.BooleanField(_('Published'), default=False)
    is_login_required = models.BooleanField(_('Login required'), default=False)
    is_permission_required = models.BooleanField(_('Permission required'), default=False)
    sites = models.ManyToManyField(Site, help_text=_('The site(s) where this pages is accessible.'),
                                   verbose_name=_('sites'))
    objects = PageManager()

    def __init__(self, *args, **kwargs):
        """Instantiate the pages object."""
        # per instance cache
        self._languages = None
        self._content = None
        self._is_first_root = None
        super(Page, self).__init__(*args, **kwargs)

    def __str__(self):
        """id string of instance"""
        return self.name

    def save(self, *args, **kwargs):
        """Override the default ``save`` method."""

        now = timezone.now()

        # update pages status flags
        if self.is_approved:
            self.is_draft = False
            # Published pages should always have a publication date
            if self.date_publication is None and not self.is_draft:
                self.date_publication = now
            if self.date_publication > now:
                self.is_published = True
            if self.date_publication_end is not None:
                if self.date_publication_end < now:
                    self.is_hidden = True
            if self.date_approved is None:
                self.date_approved = now
        else:
            if self.date_approved is not None:
                self.date_approved = None

        self.date_updated = now

        if self.template is not None:
            self.template = self.template.strip()

        # Call parent's ``save`` method
        super(Page, self).save(*args, **kwargs)

    @property
    def is_visible(self):
        """Return True if page is visible"""
        now = timezone.now()
        if self.date_publication_end:
            publication = self.date_publication < now < self.date_publication_end
        else:
            publication = self.date_publication < now
        return not self.is_draft and not self.hidden and self.is_approved and publication

    def invalidate(self):
        content = self.get_content('slug')
        for obj in content:
            cache_key = settings.PAGES_PAGE_CACHE_KEY + obj.language + ':' + obj.slug
            cache_key_version = settings.PAGES_PAGE_VERSION_KEY + obj.language + ':' + obj.slug
            cache_version = str(cache.get(cache_key_version))
            try:
                cache.incr(cache_key_version, 1)
                if settings.PAGES_CACHE_DELETE:
                    cache.delete(cache_key, version=cache_version)
            except ValueError:
                cache.set(cache_key_version, 0)

    @staticmethod
    def get_content_class(content_type):
        content_class = None
        content_type = PageContentType.objects.get(type=content_type)
        if content_type:
            try:
                content_class = getattr(pagecontenttypes, content_type.class_name)
                if not issubclass(content_class, pagecontenttypes.PageBaseContent):
                    content_class = None
            except AttributeError:
                try:
                    if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
                        if pagecontenttypes.PAGE_EXT_CONTENT_TYPES is not None:
                            for module in pagecontenttypes.PAGE_EXT_CONTENT_TYPES:
                                content_class = getattr(module, content_type.class_name)
                                if not issubclass(content_class, pagecontenttypes.PageBaseContent):
                                    content_class = None
                                else:
                                    break
                except AttributeError:
                    pass
        return content_class

    def get_template(self):
        return self.template if self.template else settings.PAGES_DEFAULT_TEMPLATE

    def get_content(self, content_type, language=None):
        content_class = self.get_content_class(content_type=content_type)
        if language is None:
            content = content_class.objects.filter(page=self.pk)
        else:
            content = content_class.objects.filter(page=self.pk, language=language)
        return content

    class Meta:
        ordering = ['tree_id', 'lft']
        get_latest_by = 'date_publication'
        app_label = 'pages'
        verbose_name = _('Page')
        verbose_name_plural = _('Pages')
        permissions = (
            ('view_page', _('Can view pages')),
        )

    class MPTTMeta:
        order_insertion_by = ['name']


@receiver(post_save, sender=Page)
def on_save(sender, instance, using, **kwargs):
    if isinstance(instance, Page):
        instance.invalidate()


@receiver(pre_delete, sender=Page)
def on_delete(sender, instance, using, **kwargs):
    if isinstance(instance, Page):
        content = instance.get_content('slug')
        for obj in content:
            slug = obj.full_slug
            cache_key = settings.PAGES_PAGE_CACHE_KEY + slug
            cache_key_version = settings.PAGES_PAGE_VERSION_KEY + slug
            cache_version = str(cache.get(cache_key_version))
            cache.delete(cache_key + cache_version)
            cache.delete(cache_key_version)


class PageUserObjectPermission(UserObjectPermissionBase):
    content_object = models.ForeignKey(Page)

    class Meta:
        app_label = 'pages'
        verbose_name = _('Page User Permissions')
        verbose_name_plural = _('Pages Users Permissions')


class PageGroupObjectPermission(GroupObjectPermissionBase):
    content_object = models.ForeignKey(Page)

    class Meta:
        app_label = 'pages'
        verbose_name = _('Page Group Permissions')
        verbose_name_plural = _('Pages Groups Permissions')


def remove_obj_perms_connected_with_user(sender, instance, **kwargs):
    filters = Q(content_type=ContentType.objects.get_for_model(instance), object_pk=instance.pk)
    UserObjectPermission.objects.filter(filters).delete()
    GroupObjectPermission.objects.filter(filters).delete()


pre_delete.connect(remove_obj_perms_connected_with_user, sender=settings.AUTH_USER_MODEL)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import image_cropping.fields
import django.utils.timezone
import pages.storage
from django.conf import settings
import markitup.fields
import mptt.fields
import pages.models.pagecontenttypes


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0001_initial'),
        ('sites', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('name', models.CharField(unique=True, max_length=200)),
                ('template', models.CharField(max_length=254, blank=True)),
                ('comment', models.TextField(max_length=254, blank=True)),
                ('date_created', models.DateTimeField(verbose_name='Created', default=django.utils.timezone.now)),
                ('date_updated', models.DateTimeField(verbose_name='Updated', default=django.utils.timezone.now)),
                ('date_approved', models.DateTimeField(verbose_name='Approved', null=True, blank=True)),
                ('date_publication', models.DateTimeField(verbose_name='Publication date', null=True, blank=True)),
                ('date_publication_end', models.DateTimeField(verbose_name='Publication end date', null=True, blank=True)),
                ('is_draft', models.BooleanField(verbose_name='Draft', default=True)),
                ('is_approved', models.BooleanField(verbose_name='Approved', default=False)),
                ('is_hidden', models.BooleanField(verbose_name='Hidden', default=False)),
                ('is_published', models.BooleanField(verbose_name='Published', default=False)),
                ('is_login_required', models.BooleanField(verbose_name='Login required', default=False)),
                ('is_permission_required', models.BooleanField(verbose_name='Permission required', default=False)),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('created_by', models.ForeignKey(related_name='page_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('parent', mptt.fields.TreeForeignKey(related_name='children', blank=True, to='pages.Page', null=True)),
                ('sites', models.ManyToManyField(verbose_name='sites', to='sites.Site', help_text='The site(s) where this pages is accessible.')),
                ('updated_by', models.ForeignKey(related_name='page_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name': 'Page',
                'permissions': (('view_page', 'Can view pages'),),
                'verbose_name_plural': 'Pages',
                'ordering': ['tree_id', 'lft'],
                'get_latest_by': 'date_publication',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
            ],
            options={
                'verbose_name': 'Content',
                'verbose_name_plural': 'Content',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageContentType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('type', models.CharField(verbose_name='Type', max_length=100)),
                ('class_name', models.CharField(verbose_name='Class', max_length=100)),
                ('admin_class_name', models.CharField(verbose_name='Admin Class', max_length=100)),
                ('is_extended', models.BooleanField(verbose_name='Extended', default=False)),
            ],
            options={
                'verbose_name': 'Content Type',
                'verbose_name_plural': 'Content Types',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageGroupObjectPermission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('content_object', models.ForeignKey(to='pages.Page')),
                ('group', models.ForeignKey(to='auth.Group')),
                ('permission', models.ForeignKey(to='auth.Permission')),
            ],
            options={
                'verbose_name': 'Page Group Permissions',
                'verbose_name_plural': 'Pages Groups Permissions',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageImageContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('type', models.CharField(verbose_name='Type', max_length=100, db_index=True)),
                ('language', models.CharField(default='en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(verbose_name='Extended?', default=False)),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(verbose_name='Created', default=django.utils.timezone.now)),
                ('date_updated', models.DateTimeField(verbose_name='Updated', default=django.utils.timezone.now)),
                ('image', image_cropping.fields.ImageCropField(null=True, storage=pages.storage.PageFileSystemStorage(), upload_to=pages.models.pagecontenttypes.make_image_upload_path, blank=True)),
                ('cropping', image_cropping.fields.ImageRatioField('image', '600x800', hide_image_field=False, verbose_name='cropping', free_crop=False, allow_fullsize=True, adapt_rotation=False, size_warning=True, help_text=None)),
                ('title', models.CharField(max_length=250, blank=True)),
                ('tags', models.CharField(max_length=250, blank=True)),
                ('created_by', models.ForeignKey(related_name='pages_pageimagecontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('updated_by', models.ForeignKey(related_name='pages_pageimagecontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name': 'Image',
                'verbose_name_plural': 'Images',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageMarkdownContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('type', models.CharField(verbose_name='Type', max_length=100, db_index=True)),
                ('language', models.CharField(default='en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(verbose_name='Extended?', default=False)),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(verbose_name='Created', default=django.utils.timezone.now)),
                ('date_updated', models.DateTimeField(verbose_name='Updated', default=django.utils.timezone.now)),
                ('text', markitup.fields.MarkupField(no_rendered_field=True, blank=True)),
                ('is_template', models.BooleanField(verbose_name='Template?', default=False)),
                ('_text_rendered', models.TextField(editable=False, blank=True)),
                ('created_by', models.ForeignKey(related_name='pages_pagemarkdowncontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('updated_by', models.ForeignKey(related_name='pages_pagemarkdowncontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name': 'Markdown',
                'verbose_name_plural': 'Markdown',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageMetaContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('type', models.CharField(verbose_name='Type', max_length=100, db_index=True)),
                ('language', models.CharField(default='en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(verbose_name='Extended?', default=False)),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(verbose_name='Created', default=django.utils.timezone.now)),
                ('date_updated', models.DateTimeField(verbose_name='Updated', default=django.utils.timezone.now)),
                ('title', models.CharField(max_length=160, blank=True)),
                ('description', models.TextField(max_length=160, blank=True)),
                ('keywords', models.CharField(max_length=200, blank=True)),
                ('is_noindex', models.BooleanField(verbose_name='NoIndex', default=False)),
                ('is_nofollow', models.BooleanField(verbose_name='NoFollow', default=False)),
                ('created_by', models.ForeignKey(related_name='pages_pagemetacontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('updated_by', models.ForeignKey(related_name='pages_pagemetacontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name': 'Meta',
                'verbose_name_plural': 'Meta',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageRedirectContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('type', models.CharField(verbose_name='Type', max_length=100, db_index=True)),
                ('language', models.CharField(default='en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(verbose_name='Extended?', default=False)),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(verbose_name='Created', default=django.utils.timezone.now)),
                ('date_updated', models.DateTimeField(verbose_name='Updated', default=django.utils.timezone.now)),
                ('redirect_to_page', models.CharField(null=True, max_length=254, blank=True)),
                ('redirect_to_url', models.URLField(null=True, max_length=254, blank=True)),
                ('is_permanent', models.BooleanField(verbose_name='Permanent', default=False)),
                ('created_by', models.ForeignKey(related_name='pages_pageredirectcontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('updated_by', models.ForeignKey(related_name='pages_pageredirectcontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name': 'Redirect',
                'verbose_name_plural': 'Redirect',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageSlugContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('type', models.CharField(verbose_name='Type', max_length=100, db_index=True)),
                ('language', models.CharField(default='en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(verbose_name='Extended?', default=False)),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(verbose_name='Created', default=django.utils.timezone.now)),
                ('date_updated', models.DateTimeField(verbose_name='Updated', default=django.utils.timezone.now)),
                ('slug', models.CharField(max_length=245)),
                ('created_by', models.ForeignKey(related_name='pages_pageslugcontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('updated_by', models.ForeignKey(related_name='pages_pageslugcontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name': 'Slug',
                'verbose_name_plural': 'Slugs',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageTextContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('type', models.CharField(verbose_name='Type', max_length=100, db_index=True)),
                ('language', models.CharField(default='en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(verbose_name='Extended?', default=False)),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(verbose_name='Created', default=django.utils.timezone.now)),
                ('date_updated', models.DateTimeField(verbose_name='Updated', default=django.utils.timezone.now)),
                ('text', models.TextField(blank=True)),
                ('is_template', models.BooleanField(verbose_name='Template?', default=False)),
                ('created_by', models.ForeignKey(related_name='pages_pagetextcontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('updated_by', models.ForeignKey(related_name='pages_pagetextcontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name': 'Text',
                'verbose_name_plural': 'Text',
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageUserObjectPermission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('content_object', models.ForeignKey(to='pages.Page')),
                ('permission', models.ForeignKey(to='auth.Permission')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Page User Permissions',
                'verbose_name_plural': 'Pages Users Permissions',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='pageslugcontent',
            unique_together=set([('language', 'page')]),
        ),
        migrations.AlterUniqueTogether(
            name='pagemetacontent',
            unique_together=set([('language', 'page')]),
        ),
        migrations.AddField(
            model_name='pagecontent',
            name='type',
            field=models.ForeignKey(to='pages.PageContentType'),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='pagecontent',
            unique_together=set([('page', 'type')]),
        ),
    ]



# -*- coding: utf-8
"""Implements PageBaseContent model"""

from __future__ import unicode_literals

from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible

from pages.conf import settings
from pages.models.pagecontenttype import get_all_content_tuple


@python_2_unicode_compatible
class PageBaseContent(models.Model):
    type = models.CharField(_('Type'), choices=get_all_content_tuple(), max_length=100, blank=False, db_index=True)
    page = models.ForeignKey('pages.Page', verbose_name=_('Page'))
    language = models.CharField(max_length=5, default=str(settings.PAGES_DEFAULT_LANGUAGE))
    sid = models.CharField(max_length=200, unique=True)
    name = models.CharField(max_length=200, blank=True, unique=True)
    is_extended = models.BooleanField(_('Extended?'), default=False)
    comment = models.CharField(max_length=250, blank=True)
    date_created = models.DateTimeField(_('Created'), default=timezone.now)
    date_updated = models.DateTimeField(_('Updated'), default=timezone.now)
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='%(app_label)s_%(class)s_creator', null=True
    )
    updated_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='%(app_label)s_%(class)s_editor', null=True
    )

    def __str__(self):
        """id string of instance"""
        return '{0:>s}'.format(self.sid)

    def count_objects(self, use_lang=True):
        if use_lang:
            count = self.__class__.objects.filter(page=self.page, language=self.language).count()
        else:
            count = self.__class__.objects.filter(page=self.page).count()
        return count

    def update_fields(self, change):
        pass

    def save(self, *args, **kwargs):
        """Override the default ``save`` method."""

        if self.pk is None:
            self.created_by = self.page.created_by
            self.update_fields(False)
            count = self.count_objects() + 1
            self.sid = '{0:>s}:{1:>s}:{2:>s}:{3:>d}'.format(
                self.language, self.page.name, self.type, count
            )
        else:
            self.update_fields(True)
        self.name = self.name if len(self.name) else self.sid
        self.updated_by = self.page.updated_by

        # Call parent's ``save`` method
        super(PageBaseContent, self).save(*args, **kwargs)

    class Meta:
        abstract = True

    class PageMeta:
        def __init__(self):
            self.context_name = 'base'
            self.multiple_per_page = True
            self.multiple_per_locale = True



from django.conf import settings
from django.conf.urls.static import static
from django.conf.urls import patterns, include, url
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin

admin.autodiscover()

from pages.conf import settings as pages_settings
from pages.views import PageDetailsView

urlpatterns = patterns('',
                       url(r'^$', PageDetailsView.as_view(), kwargs={pages_settings.PAGES_PAGE_SLUG_NAME: 'home', }),
                       url(r'^i18n/', include('django.conf.urls.i18n')),
                       url(r'^admin/', include(admin.site.urls)),
                       url(r'^markitup/', include('markitup.urls')),)

urlpatterns += i18n_patterns('',
                             url(r'^pages/', include('pages.urls')),)

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from pages.admin.page import PageAdmin
from pages.admin.pagecontent import PageContentAdmin
from pages.admin.pagecontenttype import PageContentTypeAdmin
from pages.admin.pagecontenttypes import PageSlugContentAdmin
from pages.admin.pagecontenttypes import PageRedirectContentAdmin
from pages.admin.pagecontenttypes import PageMetaContentAdmin
from pages.admin.pagecontenttypes import PageTextContentAdmin
from pages.admin.pagecontenttypes import PageImageContentAdmin



from django.contrib import admin

from pages.models import PageContent


class PageContentInline(admin.StackedInline):
    model = PageContent

    # max_num = 1
    extra = 1


class PageContentAdmin(admin.ModelAdmin):
    list_display = ['__str__']
    list_display_links = ['__str__']

    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageContent, PageContentAdmin)



from __future__ import unicode_literals
from django.contrib.auth.models import AnonymousUser

from django.core.urlresolvers import reverse
from django.template import RequestContext
from django.utils import translation

from pages.tests.base import PagesCase
from pages.models import PageSlugContent, PageMetaContent
from pages.models import PageRedirectContent
from pages.models import PageTextContent, PageMarkdownContent
from pages.models import PageImageContent
from pages.views import PageDetailsView


class TestPages(PagesCase):

    def test_page_slug_model(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')

    def test_page_meta_model(self):
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')

    def test_page_redirect_model(self):
        PageRedirectContent.objects.create(page=self.page_foo)

    def test_page_text_model(self):
        PageTextContent.objects.create(page=self.page_foo, text='test')

    def test_page_markdown_model(self):
        PageMarkdownContent.objects.create(page=self.page_foo)

    def test_page_image_model(self):
        PageImageContent.objects.create(page=self.page_foo)

    def test_page_text_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        self.page_foo.template = 'pages/page_text.html'
        self.page_foo.save()
        page_url = reverse('page_show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)

    def test_page_markdown_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageMarkdownContent.objects.create(page=self.page_foo, text='**test**')
        self.page_foo.template = 'pages/page_markdown.html'
        self.page_foo.save()
        page_url = reverse('page_show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)

    def test_page_image_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        PageImageContent.objects.create(page=self.page_foo, image='img/test.jpg', title='test')
        self.page_foo.template = 'pages/page_image.html'
        self.page_foo.save()
        page_url = reverse('page_show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)



from django import template
from django.template import Context, Template
from django.utils.html import escape

from pages.conf import settings


register = template.Library()


@register.assignment_tag()
def get_page_object(objects, sid):
    """
    **Arguments**

    ``objects``
        all objects

    ``sid`
        symbolic id for object selection

    :return selected object
    """
    selected_object = None

    for obj in objects:
        if obj.sid == sid:
            selected_object = obj
            break
    return selected_object


@register.assignment_tag(takes_context=True)
def get_page_object_by_name(context, name):
    """
    **Arguments**

    ``name`
        name for object selection

    :return selected object
    """
    selected_object = None

    for obj_type in context['page']['content']:
        for obj in context['page']['content'][obj_type]:
            if obj.name == name:
                selected_object = obj
                break
    if selected_object is None:
        for obj_type in context['page']['content']:
            for obj in context['page']['ext_content'][obj_type]:
                if obj.name == name:
                    selected_object = obj
                    break
    return selected_object


@register.assignment_tag(takes_context=True)
def get_page_object_by_id(context, object_type, oid):
    """
    **Arguments**

    ``object_type``
        object type

    ``oid``
        id for object selection

    :return selected object
    """
    if type(oid) != int:
        raise template.TemplateSyntaxError('page_object_by_id tag requires a integer argument')

    selected_object = None

    try:
        for obj in context['page']['content'][object_type]:
            sid = '{0:>s}:{1:>s}:{2:>s}:{3:>d}'.format(
                obj.language, context['page']['page'].name, obj.type, oid
            )
            if obj.sid == sid:
                selected_object = obj
                break
        if selected_object is None:
            for obj in context['page']['ext_content'][object_type]:
                sid = '{0:>s}:{1:>s}:{2:>s}:{3:>d}'.format(
                    obj.language, context['page']['page'].name, obj.type, oid
                )
                if obj.sid == sid:
                    selected_object = obj
                    break
    except KeyError:
        raise template.TemplateSyntaxError('wrong content type: {0:>s}'.format(object_type))
    return selected_object


@register.assignment_tag(takes_context=True)
def get_page_objects_by_type(context, object_type):
    """
    **Arguments**

    ``object_type``
        object type

    :return selected objects
    """
    try:
        objects = context['page']['content'][object_type]
    except KeyError:
        raise template.TemplateSyntaxError('wrong content type: {0:>s}'.format(object_type))
    return objects


@register.assignment_tag(takes_context=True)
def get_page_objects_by_ext_type(context, object_type):
    """
    **Arguments**

    ``object_type``
        object type

    :return selected objects
    """
    try:
        objects = context['page']['ext_content'][object_type]
    except KeyError:
        raise template.TemplateSyntaxError('wrong content type: {0:>s}'.format(object_type))
    return objects


@register.simple_tag(takes_context=True)
def page_text_by_id(context, oid):
    obj = get_page_object_by_id(context, 'text', oid)
    if obj is None:
        return None
    if obj.is_template and settings.PAGES_ALLOW_DJANGO_TEMPLATES:
        tpl = Template(obj.text)
        text = tpl.render(Context(context))
    else:
        text = escape(obj.text)
    return text


@register.assignment_tag(takes_context=True)
def get_page_text_by_id(context, oid):
    return page_text_by_id(context, oid)


@register.assignment_tag(takes_context=True)
def get_raw_page_text_by_id(context, oid):
    obj = get_page_object_by_id(context, 'text', oid)
    return obj.text if obj else None


@register.simple_tag(takes_context=True)
def page_markdown_by_id(context, oid):
    obj = get_page_object_by_id(context, 'markdown', oid)
    if obj is None:
        return None
    return obj.text


@register.assignment_tag(takes_context=True)
def get_page_markdown_by_id(context, oid):
    return page_markdown_by_id(context, oid)


@register.assignment_tag(takes_context=True)
def get_page_image_by_id(context, oid):
    obj = get_page_object_by_id(context, 'image', oid)
    return obj if obj else None


@register.simple_tag(takes_context=True)
def get_page_active_css_class(context, name):
    css_class = ''
    try:
        if context['page']['page'].name == name:
            css_class = settings.PAGES_PAGE_ACTIVE_CSS_CLASS
    except KeyError:
        pass
    return css_class



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'PageContentType'
        db.create_table(u'pages_pagecontenttype', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('class_name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('admin_class_name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('is_extended', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'pages', ['PageContentType'])

        # Adding model 'PageSlugContent'
        db.create_table(u'pages_pageslugcontent', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=100, db_index=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
            ('language', self.gf('django.db.models.fields.CharField')(default='en', max_length=5)),
            ('sid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200, blank=True)),
            ('is_extended', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('comment', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pageslugcontent_creator', null=True, to=orm['auth.User'])),
            ('updated_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pageslugcontent_editor', null=True, to=orm['auth.User'])),
            ('slug', self.gf('django.db.models.fields.CharField')(max_length=245)),
        ))
        db.send_create_signal(u'pages', ['PageSlugContent'])

        # Adding unique constraint on 'PageSlugContent', fields ['language', 'page']
        db.create_unique(u'pages_pageslugcontent', ['language', 'page_id'])

        # Adding model 'PageRedirectContent'
        db.create_table(u'pages_pageredirectcontent', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=100, db_index=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
            ('language', self.gf('django.db.models.fields.CharField')(default='en', max_length=5)),
            ('sid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200, blank=True)),
            ('is_extended', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('comment', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pageredirectcontent_creator', null=True, to=orm['auth.User'])),
            ('updated_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pageredirectcontent_editor', null=True, to=orm['auth.User'])),
            ('redirect_to_page', self.gf('django.db.models.fields.CharField')(max_length=254, null=True, blank=True)),
            ('redirect_to_url', self.gf('django.db.models.fields.URLField')(max_length=254, null=True, blank=True)),
            ('is_permanent', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'pages', ['PageRedirectContent'])

        # Adding model 'PageMetaContent'
        db.create_table(u'pages_pagemetacontent', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=100, db_index=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
            ('language', self.gf('django.db.models.fields.CharField')(default='en', max_length=5)),
            ('sid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200, blank=True)),
            ('is_extended', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('comment', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pagemetacontent_creator', null=True, to=orm['auth.User'])),
            ('updated_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pagemetacontent_editor', null=True, to=orm['auth.User'])),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=160, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(max_length=160, blank=True)),
            ('keywords', self.gf('django.db.models.fields.CharField')(max_length=200, blank=True)),
            ('is_noindex', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_nofollow', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'pages', ['PageMetaContent'])

        # Adding unique constraint on 'PageMetaContent', fields ['language', 'page']
        db.create_unique(u'pages_pagemetacontent', ['language', 'page_id'])

        # Adding model 'PageTextContent'
        db.create_table(u'pages_pagetextcontent', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=100, db_index=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
            ('language', self.gf('django.db.models.fields.CharField')(default='en', max_length=5)),
            ('sid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200, blank=True)),
            ('is_extended', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('comment', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pagetextcontent_creator', null=True, to=orm['auth.User'])),
            ('updated_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pagetextcontent_editor', null=True, to=orm['auth.User'])),
            ('text', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('is_template', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'pages', ['PageTextContent'])

        # Adding model 'PageMarkdownContent'
        db.create_table(u'pages_pagemarkdowncontent', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=100, db_index=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
            ('language', self.gf('django.db.models.fields.CharField')(default='en', max_length=5)),
            ('sid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200, blank=True)),
            ('is_extended', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('comment', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pagemarkdowncontent_creator', null=True, to=orm['auth.User'])),
            ('updated_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pagemarkdowncontent_editor', null=True, to=orm['auth.User'])),
            ('text', self.gf('markitup.fields.MarkupField')(no_rendered_field=True, blank=True)),
            ('is_template', self.gf('django.db.models.fields.BooleanField')(default=False)),
            (u'_text_rendered', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal(u'pages', ['PageMarkdownContent'])

        # Adding model 'PageImageContent'
        db.create_table(u'pages_pageimagecontent', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=100, db_index=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
            ('language', self.gf('django.db.models.fields.CharField')(default='en', max_length=5)),
            ('sid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200, blank=True)),
            ('is_extended', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('comment', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pageimagecontent_creator', null=True, to=orm['auth.User'])),
            ('updated_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'pages_pageimagecontent_editor', null=True, to=orm['auth.User'])),
            ('image', self.gf(u'django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('cropping', self.gf(u'django.db.models.fields.CharField')(default=u'', max_length=255, blank=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('tags', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
        ))
        db.send_create_signal(u'pages', ['PageImageContent'])

        # Adding model 'Page'
        db.create_table(u'pages_page', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('parent', self.gf('mptt.fields.TreeForeignKey')(blank=True, related_name=u'children', null=True, to=orm['pages.Page'])),
            ('template', self.gf('django.db.models.fields.CharField')(max_length=254, blank=True)),
            ('comment', self.gf('django.db.models.fields.TextField')(max_length=254, blank=True)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'page_creator', null=True, to=orm['auth.User'])),
            ('updated_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'page_editor', null=True, to=orm['auth.User'])),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_approved', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('date_publication', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('date_publication_end', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('is_draft', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('is_approved', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_hidden', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_published', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_login_required', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_permission_required', self.gf('django.db.models.fields.BooleanField')(default=False)),
            (u'lft', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'rght', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'tree_id', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'level', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
        ))
        db.send_create_signal(u'pages', ['Page'])

        # Adding M2M table for field sites on 'Page'
        m2m_table_name = db.shorten_name(u'pages_page_sites')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('page', models.ForeignKey(orm[u'pages.page'], null=False)),
            ('site', models.ForeignKey(orm[u'sites.site'], null=False))
        ))
        db.create_unique(m2m_table_name, ['page_id', 'site_id'])

        # Adding model 'PageUserObjectPermission'
        db.create_table(u'pages_pageuserobjectpermission', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('permission', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.Permission'])),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('content_object', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
        ))
        db.send_create_signal(u'pages', ['PageUserObjectPermission'])

        # Adding model 'PageGroupObjectPermission'
        db.create_table(u'pages_pagegroupobjectpermission', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('permission', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.Permission'])),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.Group'])),
            ('content_object', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
        ))
        db.send_create_signal(u'pages', ['PageGroupObjectPermission'])

        # Adding model 'PageContent'
        db.create_table(u'pages_pagecontent', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.PageContentType'])),
        ))
        db.send_create_signal(u'pages', ['PageContent'])

        # Adding unique constraint on 'PageContent', fields ['page', 'type']
        db.create_unique(u'pages_pagecontent', ['page_id', 'type_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'PageContent', fields ['page', 'type']
        db.delete_unique(u'pages_pagecontent', ['page_id', 'type_id'])

        # Removing unique constraint on 'PageMetaContent', fields ['language', 'page']
        db.delete_unique(u'pages_pagemetacontent', ['language', 'page_id'])

        # Removing unique constraint on 'PageSlugContent', fields ['language', 'page']
        db.delete_unique(u'pages_pageslugcontent', ['language', 'page_id'])

        # Deleting model 'PageContentType'
        db.delete_table(u'pages_pagecontenttype')

        # Deleting model 'PageSlugContent'
        db.delete_table(u'pages_pageslugcontent')

        # Deleting model 'PageRedirectContent'
        db.delete_table(u'pages_pageredirectcontent')

        # Deleting model 'PageMetaContent'
        db.delete_table(u'pages_pagemetacontent')

        # Deleting model 'PageTextContent'
        db.delete_table(u'pages_pagetextcontent')

        # Deleting model 'PageMarkdownContent'
        db.delete_table(u'pages_pagemarkdowncontent')

        # Deleting model 'PageImageContent'
        db.delete_table(u'pages_pageimagecontent')

        # Deleting model 'Page'
        db.delete_table(u'pages_page')

        # Removing M2M table for field sites on 'Page'
        db.delete_table(db.shorten_name(u'pages_page_sites'))

        # Deleting model 'PageUserObjectPermission'
        db.delete_table(u'pages_pageuserobjectpermission')

        # Deleting model 'PageGroupObjectPermission'
        db.delete_table(u'pages_pagegroupobjectpermission')

        # Deleting model 'PageContent'
        db.delete_table(u'pages_pagecontent')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'pages.page': {
            'Meta': {'ordering': "[u'tree_id', u'lft']", 'object_name': 'Page'},
            'comment': ('django.db.models.fields.TextField', [], {'max_length': '254', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'page_creator'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'date_approved': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_publication': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_publication_end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_approved': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_draft': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_hidden': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_login_required': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_permission_required': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            u'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'related_name': "u'children'", 'null': 'True', 'to': u"orm['pages.Page']"}),
            u'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['sites.Site']", 'symmetrical': 'False'}),
            'template': ('django.db.models.fields.CharField', [], {'max_length': '254', 'blank': 'True'}),
            u'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'updated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'page_editor'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'pages.pagecontent': {
            'Meta': {'unique_together': "((u'page', u'type'),)", 'object_name': 'PageContent'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.PageContentType']"})
        },
        u'pages.pagecontenttype': {
            'Meta': {'object_name': 'PageContentType'},
            'admin_class_name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'class_name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_extended': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'pages.pagegroupobjectpermission': {
            'Meta': {'object_name': 'PageGroupObjectPermission'},
            'content_object': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'permission': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.Permission']"})
        },
        u'pages.pageimagecontent': {
            'Meta': {'object_name': 'PageImageContent'},
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pageimagecontent_creator'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'cropping': (u'django.db.models.fields.CharField', [], {'default': "u''", 'max_length': '255', 'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': (u'django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'is_extended': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "'en'", 'max_length': '5'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'sid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            'tags': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'updated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pageimagecontent_editor'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'pages.pagemarkdowncontent': {
            'Meta': {'object_name': 'PageMarkdownContent'},
            u'_text_rendered': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pagemarkdowncontent_creator'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_extended': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_template': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "'en'", 'max_length': '5'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'sid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            'text': ('markitup.fields.MarkupField', [], {u'no_rendered_field': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'updated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pagemarkdowncontent_editor'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'pages.pagemetacontent': {
            'Meta': {'unique_together': "((u'language', u'page'),)", 'object_name': 'PageMetaContent'},
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pagemetacontent_creator'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '160', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_extended': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_nofollow': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_noindex': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "'en'", 'max_length': '5'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'sid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '160', 'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'updated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pagemetacontent_editor'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'pages.pageredirectcontent': {
            'Meta': {'object_name': 'PageRedirectContent'},
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pageredirectcontent_creator'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_extended': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_permanent': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "'en'", 'max_length': '5'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'sid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            'redirect_to_page': ('django.db.models.fields.CharField', [], {'max_length': '254', 'null': 'True', 'blank': 'True'}),
            'redirect_to_url': ('django.db.models.fields.URLField', [], {'max_length': '254', 'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'updated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pageredirectcontent_editor'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'pages.pageslugcontent': {
            'Meta': {'unique_together': "((u'language', u'page'),)", 'object_name': 'PageSlugContent'},
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pageslugcontent_creator'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_extended': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "'en'", 'max_length': '5'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'sid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            'slug': ('django.db.models.fields.CharField', [], {'max_length': '245'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'updated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pageslugcontent_editor'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'pages.pagetextcontent': {
            'Meta': {'object_name': 'PageTextContent'},
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pagetextcontent_creator'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_extended': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_template': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "'en'", 'max_length': '5'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'sid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            'text': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'updated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'pages_pagetextcontent_editor'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'pages.pageuserobjectpermission': {
            'Meta': {'object_name': 'PageUserObjectPermission'},
            'content_object': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'permission': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.Permission']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'sites.site': {
            'Meta': {'ordering': "(u'domain',)", 'object_name': 'Site', 'db_table': "u'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['pages']



# -*- coding: utf-8
"""Implements page content models"""

from __future__ import unicode_literals
import hashlib
import os
import base64
import uuid
import importlib

from django.core.urlresolvers import reverse
from django.db import models
from django.db.models.signals import post_delete
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify

from image_cropping import ImageCropField
from image_cropping import ImageRatioField
from markitup.fields import MarkupField

from pages.conf import settings
from pages.models.pagebasecontent import PageBaseContent
from pages.storage import PageFileSystemStorage


PAGE_EXT_CONTENT_TYPES = []

if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
    if settings.PAGES_PAGE_EXT_CONTENT_TYPES is not None:
        try:
            for content_type in settings.PAGES_PAGE_EXT_CONTENT_TYPES:
                PAGE_EXT_CONTENT_TYPES.append(importlib.import_module(content_type))
        except ImportError as e:
            raise Exception('Extended content type import error: {0}'.format(e))


class PageSlugContent(PageBaseContent):
    slug = models.CharField(max_length=245)
    objects = models.Manager()

    @property
    def full_slug(self):
        return self.language + '/' + self.slug

    def update_fields(self, change):
        if not change:
            self.type = 'slug'
            if not self.slug:
                self.slug = slugify(self.page.name)

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Slug')
        verbose_name_plural = _('Slugs')
        unique_together = ('language', 'page',)

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'slug'
        multiple_per_page = True
        multiple_per_locale = False


class PageRedirectContent(PageBaseContent):
    redirect_to_page = models.CharField(max_length=254, null=True, blank=True)
    redirect_to_url = models.URLField(max_length=254, null=True, blank=True)
    is_permanent = models.BooleanField(_('Permanent'), default=False)

    def update_fields(self, change):
        if not change:
            self.type = 'redirect'

    def get_redirect_url(self, request):
        url = None
        if self.redirect_to_page:
            url = request.build_absolute_uri(self.redirect_to_page)
        else:
            if self.redirect_to_url:
                url = self.redirect_to_url
        return url

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Redirect')
        verbose_name_plural = _('Redirect')

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'redirect'
        multiple_per_page = True
        multiple_per_locale = False


class PageMetaContent(PageBaseContent):
    title = models.CharField(max_length=160, blank=True)
    description = models.TextField(max_length=160, blank=True)
    keywords = models.CharField(max_length=200, blank=True)
    is_noindex = models.BooleanField(_('NoIndex'), default=False)
    is_nofollow = models.BooleanField(_('NoFollow'), default=False)
    objects = models.Manager()

    def update_fields(self, change):
        if not change:
            self.type = 'meta'

    @property
    def robots(self):
        """Return values for robots html meta key"""
        r = 'noindex' if self.is_noindex else 'index'
        r += ','
        r += 'nofollow' if self.is_nofollow else 'follow'
        return r

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Meta')
        verbose_name_plural = _('Meta')
        unique_together = ('language', 'page',)

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'meta'
        multiple_per_page = True
        multiple_per_locale = False


class PageTextContent(PageBaseContent):
    text = models.TextField(blank=True)
    is_template = models.BooleanField(_('Template?'), default=False)
    objects = models.Manager()

    def update_fields(self, change):
        if not change:
            self.type = 'text'

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Text')
        verbose_name_plural = _('Text')

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'text'
        multiple_per_page = True
        multiple_per_locale = True


class PageMarkdownContent(PageBaseContent):
    text = MarkupField(blank=True)
    is_template = models.BooleanField(_('Template?'), default=False)
    objects = models.Manager()

    def update_fields(self, change):
        if not change:
            self.type = 'markdown'

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Markdown')
        verbose_name_plural = _('Markdown')

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'markdown'
        multiple_per_page = True
        multiple_per_locale = True


image_file_storage = PageFileSystemStorage()


def make_image_upload_path(instance, filename, prefix=False):
    """Generate upload path and new filename for pages image"""

    # store original filename
    instance.filename = filename
    # always generate unique name, for security reason
    file_uuid = uuid.uuid4().hex

    # generate short sha1 based hash
    name_sha = hashlib.sha1(file_uuid).digest()
    name_hash = base64.urlsafe_b64encode(name_sha[:9]).decode('utf-8')

    return u'{path}/{sub0}/{sub1}/{sub2}/{name}.{ext}'.format(
        path=settings.PAGES_IMAGE_DIR,
        sub0=file_uuid[0:2],
        sub1=file_uuid[7:9],
        sub2=file_uuid[12:14],
        name=name_hash,
        ext=os.path.splitext(filename)[1].strip('.').lower()
    )


class PageImageContent(PageBaseContent):
    image = ImageCropField(blank=True, null=True, upload_to=make_image_upload_path, storage=image_file_storage)
    cropping = ImageRatioField('image', '{0:>s}x{1:>s}'.format(
        str(settings.PAGES_IMAGE_WIDTH_MAX), str(settings.PAGES_IMAGE_HEIGHT_MAX)), allow_fullsize=True)
    title = models.CharField(max_length=250, blank=True)
    tags = models.CharField(max_length=250, blank=True)
    objects = models.Manager()

    def update_fields(self, change):
        if not change:
            self.type = 'image'

    def image_cropping_link(self):
        """Return image cropping link"""
        change_url = reverse('admin:pages_pageimagecontent_change', args=(self.id,))
        return '<a href="{0:>s}" onclick="return showAddAnotherPopup(this);">{1:>s}</a>'.format(change_url, _('CROP'))

    image_cropping_link.short_description = _('Cropping')
    image_cropping_link.allow_tags = True

    class Meta(PageBaseContent.Meta):
        app_label = 'pages'
        verbose_name = _('Image')
        verbose_name_plural = _('Images')

    class PageMeta(PageBaseContent.PageMeta):
        context_name = 'images'
        multiple_per_page = True
        multiple_per_locale = True


# Signals handler for deleting files after object record deleted
# In Django 1.3+, delete a record not remove the associated files
@receiver(post_delete, sender=PageImageContent)
def delete_image(sender, **kwargs):
    """Automatically delete image file when records removed."""
    if settings.PAGES_DELETE_IMAGE_FILE:
        image = kwargs.get('instance')
        if image.image is not None:
            try:
                image.image.storage.delete(image.image.path)
            except Exception:
                pass



from django import template

from pages.templatetags.pages_tags import get_page_object_by_id

register = template.Library()


@register.simple_tag(takes_context=True)
def page_video_by_id(context, oid):
    obj = get_page_object_by_id(context, 'video', oid)
    if obj is None:
        return None
    return obj.text


@register.assignment_tag(takes_context=True)
def get_page_video_by_id(context, oid):
    return page_video_by_id(context, oid)



"""Django urls for content management system."""

from django.conf.urls import patterns, url

from pages.views import PageDetailsView

urlpatterns = patterns('pages.views',
    # pages
    url(r'^(?P<slug>[-\w]+)/$', PageDetailsView.as_view(), name='page_show'),
)



# -*- coding: utf-8
"""Implements PageContentType model"""

from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from pages.managers import ContentTypeManager


def get_all_content_tuple():
    content_types = []
    try:
        types = PageContentType.objects.all()
        for i in types:
            content_types.append((i.type, i.type))
    except Exception:
        # Probably PageContentType table not populated.
        pass
    return tuple(content_types)


@python_2_unicode_compatible
class PageContentType(models.Model):
    type = models.CharField(_('Type'), max_length=100, blank=False)
    class_name = models.CharField(_('Class'), max_length=100, blank=False)
    admin_class_name = models.CharField(_('Admin Class'), max_length=100, blank=False)
    is_extended = models.BooleanField('Extended', default=False)
    objects = ContentTypeManager()

    def __str__(self):
        """id string of instance"""
        return '{0}'.format(self.type)

    class Meta:
        app_label = 'pages'
        verbose_name = _('Content Type')
        verbose_name_plural = _('Content Types')



import django

from pages.conf import settings

if django.VERSION[:2] >= (1, 7):
    from django.core.cache import caches
    cache = caches[settings.PAGES_CACHE_BACKEND]
else:
    from django.core import cache as cache_module
    cache = cache_module.get_cache(settings.PAGES_CACHE_BACKEND)



import os

SITE_ID = 1

BASE_DIR = os.path.dirname(__file__)

from django.utils.translation import ugettext_lazy as _

ROOT_URLCONF = 'urls'
SECRET_KEY = 'secretkey'
SITE_ROOT = os.path.dirname(os.path.abspath(__file__))

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

LANGUAGES = (
    ('en', _('English')),
    ('ru', _('Russian')),
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.sessions',
    'django.contrib.contenttypes',
    'django.contrib.admin',
    'django.contrib.messages',
    'django.contrib.sites',
    'guardian',
    'mptt',
    'markitup',
    'easy_thumbnails',
    'image_cropping',
    'pages',
)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test.db',                      # Or path to database file if using sqlite3.
        'USER': '',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware'
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.messages.context_processors.messages',
)

# Templates settings
TEMPLATE_DIRS = tuple([os.path.join(BASE_DIR, app_name, 'templates') for app_name in INSTALLED_APPS])
TEMPLATE_DIRS += tuple([os.path.join(BASE_DIR, 'templates')])


# django-guardian settings
AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
    'guardian.backends.ObjectPermissionBackend',
)

ANONYMOUS_USER_ID = -1

# MarkItUp settings
MARKITUP_FILTER = ('markdown.markdown', {'safe_mode': False, 'extensions': ['extra', 'meta', 'nl2br']})



"""
Django settings for django_pages_cms_test project.

For more information on this file, see
https://docs.djangoproject.com/en/dev/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/dev/ref/settings/
"""

# debug control
# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True
TEMPLATE_DEBUG = True
CACHE_DEBUG = True
SESSION_DEBUG = True
DEBUG_TOOLBAR = True

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

from django.utils.translation import ugettext_lazy as _

SITE_ID = 1

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/dev/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '!at4)qus9dqcgv8t_#+6%uz-ispry)mhdd#-fcuake0i8hmz16'

ALLOWED_HOSTS = []

# Internationalization
# https://docs.djangoproject.com/en/dev/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

LANGUAGES = (
    ('en', _('English')),
    ('ru', _('Russian')),
)


# List of callables that know how to import templates from various sources.
if TEMPLATE_DEBUG:
    TEMPLATE_LOADERS = (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
        'django.template.loaders.eggs.Loader',
    )
else:
    # if debug disabled we use cached template loaders
    TEMPLATE_LOADERS = (
        ('django.template.loaders.cached.Loader', (
            'django.template.loaders.filesystem.Loader',
            'django.template.loaders.app_directories.Loader',
            'django.template.loaders.eggs.Loader',
        )),
    )


# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    # should be enabled only for django < 1.7
    # 'south',
    'mptt',
    'guardian',
    'markitup',
    'easy_thumbnails',
    'image_cropping',
    'pages',
    'extpages',
)


MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)


# Templates settings
TEMPLATE_DIRS = tuple([os.path.join(BASE_DIR, app_name, 'templates') for app_name in INSTALLED_APPS])
TEMPLATE_DIRS += tuple([os.path.join(BASE_DIR, 'templates')])

ROOT_URLCONF = 'django_pages_cms_test.urls'

WSGI_APPLICATION = 'django_pages_cms_test.wsgi.application'


# Database
# https://docs.djangoproject.com/en/dev/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Absolute filesystem path to the directory that will hold user-uploaded files.
MEDIA_ROOT = os.path.join(BASE_DIR, 'web/media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
MEDIA_URL = '/media/'

# Absolute filesystem path to the directory that will hold user-uploaded files.
STATIC_ROOT = os.path.join(BASE_DIR, 'web/static')

# Static files (CSS, JavaScript, Images)
STATIC_URL = '/static/'

# guardian settings
AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',  # this is default
    'guardian.backends.ObjectPermissionBackend',
)

ANONYMOUS_USER_ID = -1


# easy_thumbnails
# save thumbnail images to a sub-directory relative to the source image.
THUMBNAIL_DEBUG = DEBUG
THUMBNAIL_SUBDIR = 'thumbs'
THUMBNAIL_QUALITY = 95

from easy_thumbnails.conf import settings as thumbnail_settings

THUMBNAIL_PROCESSORS = ('image_cropping.thumbnail_processors.crop_corners', ) + thumbnail_settings.THUMBNAIL_PROCESSORS

# image_cropping
IMAGE_CROPPING_THUMB_SIZE = (400, 400)
IMAGE_CROPPING_SIZE_WARNING = True

# remove south settings for django 1.7+
SOUTH_MIGRATION_MODULES = {
    'easy_thumbnails': 'easy_thumbnails.south_migrations',
    'pages': 'pages.south_migrations',
}


# MarkItUp settings
MARKITUP_MEDIA_URL = STATIC_URL
MARKITUP_FILTER = ('markdown.markdown', {'safe_mode': False, 'extensions': ['extra', 'meta', 'nl2br']})
MARKITUP_SET = 'markitup/sets/markdown'
MARKITUP_SKIN = 'markitup/skins/simple'
MARKITUP_AUTO_PREVIEW = True



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'PageVideoContent'
        db.create_table(u'extpages_pagevideocontent', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=100, db_index=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['pages.Page'])),
            ('language', self.gf('django.db.models.fields.CharField')(default='en', max_length=5)),
            ('sid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200, blank=True)),
            ('is_extended', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('comment', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('date_updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'extpages_pagevideocontent_creator', null=True, to=orm['auth.User'])),
            ('updated_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'extpages_pagevideocontent_editor', null=True, to=orm['auth.User'])),
            ('video', self.gf('embed_video.fields.EmbedVideoField')(max_length=200, blank=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=160, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(max_length=160, blank=True)),
        ))
        db.send_create_signal(u'extpages', ['PageVideoContent'])


    def backwards(self, orm):
        # Deleting model 'PageVideoContent'
        db.delete_table(u'extpages_pagevideocontent')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'extpages.pagevideocontent': {
            'Meta': {'object_name': 'PageVideoContent'},
            'comment': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'extpages_pagevideocontent_creator'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '160', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_extended': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "'en'", 'max_length': '5'}),
            'sid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['pages.Page']"}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '160', 'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'updated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'extpages_pagevideocontent_editor'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'video': ('embed_video.fields.EmbedVideoField', [], {'max_length': '200', 'blank': 'True'})
        },
        u'pages.page': {
            'Meta': {'ordering': "[u'tree_id', u'lft']", 'object_name': 'Page'},
            'comment': ('django.db.models.fields.TextField', [], {'max_length': '254', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'page_creator'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'date_approved': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'date_publication': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_publication_end': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_approved': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_draft': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_hidden': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_login_required': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_permission_required': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            u'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'related_name': "u'children'", 'null': 'True', 'to': u"orm['pages.Page']"}),
            u'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['sites.Site']", 'symmetrical': 'False'}),
            'template': ('django.db.models.fields.CharField', [], {'max_length': '254', 'blank': 'True'}),
            u'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'updated_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'page_editor'", 'null': 'True', 'to': u"orm['auth.User']"})
        },
        u'sites.site': {
            'Meta': {'ordering': "(u'domain',)", 'object_name': 'Site', 'db_table': "u'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['extpages']



import os
from setuptools import setup, find_packages
from pages import get_version as get_package_version


README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
REQUIREMENTS = os.path.join(os.path.dirname(__file__), 'requirements.txt')
reqs = open(REQUIREMENTS).read().splitlines()


# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

try:
    import importlib
except ImportError:
    reqs.append('importlib')

setup(
    name='django-pages-cms',
    version=get_package_version(),
    packages=find_packages(),
    include_package_data=True,
    install_requires=reqs,
    license='BSD',
    description='Simple content management system for Django 1.6+',
    long_description=README,
    url='https://github.com/dlancer/django-pages-cms',
    author='dlancer',
    author_email='dmdpost@gmail.com',
    maintainer='dlancer',
    maintainer_email='dmdpost@gmail.com',
    zip_safe=False,
    use_2to3=True,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Development Status :: 3 - Alpha',
    ],)



from django.conf import settings
from appconf import AppConf


class PagesAppConf(AppConf):
    DEFAULT_LANGUAGE = getattr(settings, 'PAGES_DEFAULT_LANGUAGE', 'en')
    FALLBACK_LANGUAGE = getattr(settings, 'PAGES_FALLBACK_LANGUAGE', 'en')

    ALLOW_DJANGO_TEMPLATES = getattr(settings, 'PAGES_ALLOW_DJANGO_TEMPLATES', False)

    USE_SITE_ID = getattr(settings, 'PAGES_USE_SITE_ID', False)
    HIDE_SITES = getattr(settings, 'PAGES_HIDE_SITES', True)
    DEFAULT_TEMPLATE = getattr(settings, 'PAGES_DEFAULT_TEMPLATE', 'pages/page.html')
    RAISE_403 = getattr(settings, 'PAGES_RAISE_403', True)
    RENDER_403 = getattr(settings, 'PAGES_RENDER_403', False)
    TEMPLATE_403 = getattr(settings, 'PAGES_TEMPLATE_403', 'pages/403.html')

    PAGE_SLUG_NAME = getattr(settings, 'PAGES_PAGE_SLUG_NAME', 'slug')
    CACHE_BACKEND = getattr(settings, 'PAGES_CACHE_BACKEND', 'default')
    CACHE_PREFIX = getattr(settings, 'PAGES_CACHE_PREFIX', 'pages_')
    CACHE_DELETE = getattr(settings, 'PAGES_CACHE_DELETE', False)
    PAGE_CACHE_KEY = getattr(settings, 'PAGES_PAGE_CACHE_KEY', 'page_cache_key_')
    PAGE_VERSION_KEY = getattr(settings, 'PAGES_PAGE_VERSION_KEY', 'page_version_key_')
    PAGE_CACHE_TIMEOUT = getattr(settings, 'PAGES_PAGE_CACHE_TIMEOUT', 31536000)  # timeout 1 year by default

    PAGE_HTTP_MAX_AGE = getattr(settings, 'PAGES_PAGE_CACHE_TIMEOUT', 3600)  # timeout 1 hour by default

    FILE_LOCATION = getattr(settings, 'PAGES_FILE_LOCATION', settings.MEDIA_ROOT)
    FILE_UPLOAD_PERMISSIONS = getattr(
        settings, 'PAGES_FILE_UPLOAD_PERMISSIONS', settings.FILE_UPLOAD_PERMISSIONS)

    if getattr(settings, 'FILE_UPLOAD_DIRECTORY_PERMISSIONS', False):
        FILE_UPLOAD_DIRECTORY_PERMISSIONS = getattr(
            settings, 'PAGES_FILE_UPLOAD_DIRECTORY_PERMISSIONS', settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS)

    IMAGE_DIR = getattr(settings, 'PAGES_IMAGE_DIR', 'pages')
    IMAGE_WIDTH_MAX = getattr(settings, 'PAGES_IMAGE_WIDTH_MAX', 600)
    IMAGE_HEIGHT_MAX = getattr(settings, 'PAGES_IMAGE_HEIGHT_MAX', 800)
    DELETE_IMAGE_FILE = getattr(settings, 'PAGES_DELETE_IMAGE_FILE', False)

    PAGE_USE_EXT_CONTENT_TYPES = getattr(settings, 'PAGES_PAGE_USE_EXT_CONTENT_TYPES', False)
    PAGE_EXT_CONTENT_TYPES = getattr(settings, 'PAGES_PAGE_EXT_CONTENT_TYPES', None)
    PAGE_EXT_CONTENT_INLINES = getattr(settings, 'PAGES_PAGE_EXT_CONTENT_INLINES', None)

    PAGE_ACTIVE_CSS_CLASS = getattr(settings, 'PAGES_PAGE_ACTIVE_CSS_CLASS', 'active')

    class Meta:
        prefix = 'pages'



"""Implements admin interface for page content"""

from django.contrib import admin

from image_cropping import ImageCroppingMixin

from pages.models import PageSlugContent
from pages.models import PageRedirectContent
from pages.models import PageMetaContent
from pages.models import PageTextContent
from pages.models import PageImageContent
from pages.models import PageMarkdownContent


class PageSlugContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageSlugContent, PageSlugContentAdmin)


class PageSlugContentInline(admin.StackedInline):
    model = PageSlugContent
    # max_num = 1
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated', )
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('slug', ),
            ('comment', ),
        ]}),
    ]


class PageRedirectContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by')
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageRedirectContent, PageRedirectContentAdmin)


class PageRedirectContentInline(admin.StackedInline):
    model = PageRedirectContent
    # max_num = 1
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('redirect_to_page', ),
            ('redirect_to_url', ),
            ('is_permanent', ),
            ('comment', ),
        ]}),
    ]


class PageMetaContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ['type', 'sid', 'is_extended', 'created_by', 'updated_by']
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageMetaContent, PageMetaContentAdmin)


class PageMetaContentInline(admin.StackedInline):
    model = PageMetaContent
    # max_num = 1
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('title', ),
            ('keywords', ),
            ('description', ),
            ('is_noindex', 'is_nofollow', ),
            ('comment', ),
        ]}),
    ]


class PageTextContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageTextContent, PageTextContentAdmin)


class PageTextContentInline(admin.StackedInline):
    model = PageTextContent
    # max_num = 1
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated', )
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('text', ),
            ('is_template', ),
            ('comment', ),
        ]}),
    ]


class PageImageContentAdmin(ImageCroppingMixin, admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageImageContent, PageImageContentAdmin)


class PageImageContentInline(admin.StackedInline):
    model = PageImageContent
    # max_num = 1
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    readonly_fields = ['image_cropping_link']
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('image', ),
            ('image_cropping_link', ),
            ('title', ),
            ('tags', ),
            ('comment', ),
        ]}),
    ]


class PageMarkdownContentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
    list_display_links = ['__str__']

    exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
    save_on_top = True
    actions_on_bottom = True

admin.site.register(PageMarkdownContent, PageMarkdownContentAdmin)


class PageMarkdownContentInline(admin.StackedInline):
    model = PageMarkdownContent
    # max_num = 1
    extra = 1
    exclude = ('sid', 'created_by', 'updated_by', 'date_created', 'date_updated', )
    fieldsets = [
        (None, {'fields': [
            ('language', ),
            ('name', ),
            ('text', ),
            ('is_template', ),
            ('comment', ),
        ]}),
    ]
